@@ -25,6 +25,12 @@ import mysql.fabric.persistence as _persistence
 import mysql.fabric.utils as _utils
 
 from mysql.fabric.server import MySQLServer, Group
+from mysql.fabric.sharding_datatype import (
+    HashShardingHandler,
+    RangeShardingIntegerHandler,
+    RangeShardingStringHandler,
+    RangeShardingDateTimeHandler
+)
 
 class ShardMapping(_persistence.Persistable):
     """Represents the mapping between the sharding scheme and the table
@@ -76,7 +82,9 @@ class ShardMapping(_persistence.Persistable):
                             "table_name VARCHAR(64) NOT NULL, "
                             "column_name VARCHAR(64) NOT NULL, "
                             "PRIMARY KEY (table_name, column_name), "
-                            "INDEX(shard_mapping_id))")
+                            "INDEX(shard_mapping_id)) "
+                            "DEFAULT CHARSET=utf8"
+    )
 
     DUMP_SHARD_TABLES = (
                             "SELECT "
@@ -103,11 +111,17 @@ class ShardMapping(_persistence.Persistable):
         "ORDER BY r.shard_id, t.table_name, t.column_name, r.lower_bound"
     )
 
-    CREATE_SHARD_MAPPING_DEFN = ("CREATE TABLE shard_maps"
-                                 "(shard_mapping_id INT AUTO_INCREMENT "
-                                 "NOT NULL PRIMARY KEY, "
-                                 "type_name ENUM('RANGE', 'HASH') NOT NULL, "
-                                 "global_group VARCHAR(64))")
+    CREATE_SHARD_MAPPING_DEFN = (
+        "CREATE TABLE shard_maps"
+         "("
+         "shard_mapping_id INT AUTO_INCREMENT NOT NULL PRIMARY KEY, "
+         "type_name ENUM "
+         "("
+         "'RANGE','RANGE_INTEGER','RANGE_STRING','RANGE_DATETIME','HASH'"
+         ") NOT NULL, "
+         "global_group VARCHAR(64)"
+         ") DEFAULT CHARSET=utf8"
+    )
 
     DUMP_SHARD_MAPS = (
                             "SELECT "
@@ -261,8 +275,7 @@ class ShardMapping(_persistence.Persistable):
 
         sharding_type = sharding_type.upper()
         cur = persister.exec_stmt(ShardMapping.LIST_SHARD_MAPPINGS,
-                                  {"raw" : False,
-                                  "fetch" : False,
+                                  {"fetch" : False,
                                   "params" : (sharding_type,)})
         rows = cur.fetchall()
 
@@ -318,8 +331,7 @@ class ShardMapping(_persistence.Persistable):
         """
         cur = persister.exec_stmt(
                                   ShardMapping.SELECT_SHARD_MAPPING,
-                                  {"raw" : False,
-                                  "fetch" : False,
+                                  {"fetch" : False,
                                   "params" : (table_name,)})
         row = cur.fetchone()
         if row:
@@ -344,8 +356,7 @@ class ShardMapping(_persistence.Persistable):
 
         cur = persister.exec_stmt(
                                   ShardMapping.SELECT_SHARD_MAPPING_BY_ID,
-                                  {"raw" : False,
-                                  "fetch" : False,
+                                  {"fetch" : False,
                                   "params" : (shard_mapping_id,)})
         rows = cur.fetchall()
 
@@ -367,9 +378,7 @@ class ShardMapping(_persistence.Persistable):
         :return: A list containing the shard mapping definition parameters.
         """
         row = persister.exec_stmt(ShardMapping.SELECT_SHARD_MAPPING_DEFN,
-                                  {"raw" : False,
-                                  "fetch" : True,
-                                  "params" : (shard_mapping_id,)})
+                                  {"params" : (shard_mapping_id,)})
         if row:
             #There is no abstraction for a shard mapping definition. A
             #shard mapping definition is just a triplet of
@@ -385,9 +394,7 @@ class ShardMapping(_persistence.Persistable):
         :param persister: A valid handle to the state store.
         :return: A list containing the shard mapping definitions.
         """
-        rows = persister.exec_stmt(ShardMapping.LIST_SHARD_MAPPING_DEFN,
-                                  {"raw" : False,
-                                  "fetch" : True})
+        rows = persister.exec_stmt(ShardMapping.LIST_SHARD_MAPPING_DEFN)
         if rows is not None:
             return rows
 
@@ -395,17 +402,17 @@ class ShardMapping(_persistence.Persistable):
 
     @staticmethod
     def lookup_shard_mapping_id(group_id, persister=None):
-      """Return the shard mapping associated with a group.
+        """Return the shard mapping associated with a group.
 
-      :group_id: Group identification.
-      :param persister: A valid handle to the state store.
-      :return: Shard mapping associated to a group..
-      """
-      rows = persister.exec_stmt(ShardMapping.QUERY_SHARD_MAPPING_PER_GROUP, 
-          {"params": (group_id, ), "raw" : False, "fetch" : True}
-      )
-      if rows:
-          return rows[0][0]
+        :group_id: Group identification.
+        :param persister: A valid handle to the state store.
+        :return: Shard mapping associated to a group..
+        """
+        rows = persister.exec_stmt(ShardMapping.QUERY_SHARD_MAPPING_PER_GROUP,
+          {"params": (group_id, )}
+        )
+        if rows:
+            return rows[0][0]
 
     @staticmethod
     def define(type_name, global_group_id, persister=None):
@@ -444,6 +451,21 @@ class ShardMapping(_persistence.Persistable):
             {"params":(shard_mapping_id, table_name, column_name)})
         return ShardMapping.fetch(table_name)
 
+    @staticmethod
+    def add_constraints(persister=None):
+        """Add the constraints on the tables that stores the list of registered
+        sharded tables.
+
+        :param persister: The DB server that can be used to access the
+                          state store.
+        """
+        persister.exec_stmt(
+            ShardMapping.ADD_FOREIGN_KEY_CONSTRAINT_GLOBAL_GROUP
+        )
+        persister.exec_stmt(
+            ShardMapping.ADD_FOREIGN_KEY_CONSTRAINT_SHARD_MAPPING_ID
+        )
+
     @staticmethod
     def dump_shard_tables(version=None, patterns="", persister=None):
         """Return the list of shard tables that belong to the shard mappings
@@ -455,9 +477,6 @@ class ShardMapping(_persistence.Persistable):
         :return: A list of registered tables using the provided shard mapping
                 pattern.
         """
-        #This is used to store the consolidated list of shard mappings
-        #that will be returned.
-        result_shard_tables_list = []
 
         #This stores the pattern that will be passed to the LIKE MySQL
         #command.
@@ -483,15 +502,7 @@ class ShardMapping(_persistence.Persistable):
             #a database and table name.
             for row in rows:
                 database, table = _utils.split_database_table(row[0])
-                result_shard_tables_list.append(
-                    (
-                        database,
-                        table,
-                        row[1],
-                        row[2]
-                    )
-                )
-        return _utils.wrap_output(result_shard_tables_list)
+                yield (database, table, row[1], row[2] )
 
     @staticmethod
     def dump_sharding_info(version=None, patterns="", persister=None):
@@ -503,9 +514,6 @@ class ShardMapping(_persistence.Persistable):
         :param persister: Persister to persist the object to.
         :return: The sharding information for all the tables passed in patterns.
         """
-        #This is used to store the consolidated list of shard mappings
-        #that will be returned.
-        result_shard_tables_list = []
 
         #This stores the pattern that will be passed to the LIKE MySQL
         #command.
@@ -531,19 +539,16 @@ class ShardMapping(_persistence.Persistable):
             #a database and table name.
             for row in rows:
                 database, table = _utils.split_database_table(row[0])
-                result_shard_tables_list.append(
-                    (
-                        database,
-                        table,
-                        row[1],  # column_name
-                        row[2],  # lower_bound
-                        row[3],  # shard_id
-                        row[4],  # group_id
-                        row[5],  # global_group
-                        row[6],  # type
-                    )
+                yield (
+                    database,
+                    table,
+                    row[1],     # column_name
+                    row[2],     # lower_bound
+                    row[3],     # shard_id
+                    row[4],     # type_name
+                    row[5],     # group_id
+                    row[6],     # global_group
                 )
-        return _utils.wrap_output(result_shard_tables_list)
 
     @staticmethod
     def dump_shard_maps(version=None, patterns="", persister=None):
@@ -555,9 +560,6 @@ class ShardMapping(_persistence.Persistable):
         :param persister: Persister to persist the object to.
         :return: A list of shard mappings matching the provided pattern.
         """
-        #This is used to store the consolidated list of shard mappings
-        #that will be returned.
-        result_shard_maps = []
 
         #This stores the pattern that will be passed to the LIKE MySQL
         #command.
@@ -580,8 +582,7 @@ class ShardMapping(_persistence.Persistable):
                                   {"fetch" : False, "params":(like_pattern,)})
             rows = cur.fetchall()
             for row in rows:
-                result_shard_maps.append((row[0], row[1], row[2]))
-        return _utils.wrap_output(result_shard_maps)
+                yield (row[0], row[1], row[2])
 
 class Shards(_persistence.Persistable):
     """Contains the mapping between the Shard ID and the Group ID
@@ -602,8 +603,17 @@ class Shards(_persistence.Persistable):
     state - Indicates whether a given shard is ENABLED or DISABLED.
     """
 
+    #Tuple stores the list of valid Range Sharding Types.
+    VALID_RANGE_SHARDING_TYPES = (
+        'RANGE', 'RANGE_INTEGER','RANGE_STRING', 'RANGE_DATETIME',
+    )
+
+    #Tuple stores the list of valid Hash Sharding Types.
+    VALID_HASH_SHARDING_TYPES = ("HASH",)
+
     #Tuple stores the list of valid shard mapping types.
-    VALID_SHARDING_TYPES = ('RANGE', 'HASH')
+    VALID_SHARDING_TYPES = \
+        VALID_RANGE_SHARDING_TYPES + VALID_HASH_SHARDING_TYPES
 
     #Valid states of the shards
     VALID_SHARD_STATES = ("ENABLED", "DISABLED")
@@ -612,7 +622,9 @@ class Shards(_persistence.Persistable):
     CREATE_SHARDS = ("CREATE TABLE shards ("
                     "shard_id INT AUTO_INCREMENT NOT NULL PRIMARY KEY, "
                     "group_id VARCHAR(64) NOT NULL, "
-                    "state ENUM('DISABLED', 'ENABLED') NOT NULL)")
+                    "state ENUM('DISABLED', 'ENABLED') NOT NULL) "
+                    "DEFAULT CHARSET=utf8"
+    )
 
     #Create the referential integrity constraint with the groups table.
     ADD_FOREIGN_KEY_CONSTRAINT_GROUP_ID = \
@@ -796,9 +808,6 @@ class Shards(_persistence.Persistable):
         :return: A list of shard indexes belonging to the shard mappings that
                 match the provded patterns.
         """
-        #This is used to store the consolidated list of shard mappings
-        #that will be returned.
-        result_shard_indexes_list = []
 
         #This stores the pattern that will be passed to the LIKE MySQL
         #command.
@@ -821,173 +830,12 @@ class Shards(_persistence.Persistable):
                                   {"fetch" : False, "params":(like_pattern,)})
             rows = cur.fetchall()
             for row in rows:
-                result_shard_indexes_list.append(
-                    (
-                        row[0], row[1], row[2], row[3]
-                    )
+                yield (
+                    row[0],
+                    row[1],
+                    row[2],
+                    row[3]
                 )
-        return _utils.wrap_output(result_shard_indexes_list)
-
-class RangeShardingIntegerHandler(object):
-    """Contains the members that are required to handle a sharding definition
-    based on an INTEGER datatype.
-    """
-
-   #Select the server corresponding to the RANGE to which a given key
-    #belongs. The query either selects the least lower_bound that is larger
-    #than a given key or selects the largest lower_bound and insert the key
-    #in that shard.
-    LOOKUP_KEY = (
-        "SELECT "
-        "sr.shard_mapping_id, "
-        "sr.lower_bound, "
-        "s.shard_id "
-        "FROM "
-        "shard_ranges AS sr, shards AS s "
-        "WHERE %s >= CAST(lower_bound AS SIGNED) "
-        "AND sr.shard_mapping_id = %s "
-        "AND s.shard_id = sr.shard_id "
-        "ORDER BY CAST(lower_bound AS SIGNED) DESC "
-        "LIMIT 1"
-    )
-
-    #Select the UPPER BOUND for a given LOWER BOUND value.
-    SELECT_UPPER_BOUND = (
-        "SELECT lower_bound FROM "
-        "shard_ranges "
-        "WHERE  CAST(lower_bound AS SIGNED) > %s AND shard_mapping_id = %s "
-        "ORDER BY  CAST(lower_bound AS SIGNED) ASC LIMIT 1"
-    )
-
-    @staticmethod
-    def is_valid_lower_bound(lower_bound):
-        """Verify if the given value is a valid INTEGER lower bound.
-
-        :param lower_bound: The value that needs to be verified.
-        """
-        try:
-            int(lower_bound)
-            return True
-        except ValueError:
-            return False
-
-    @staticmethod
-    def split_value(lower_bound, upper_bound):
-        """Find the middle value of a sharding range that can be used when
-        the split value is not explicitly specified.
-
-        :param lower_bound: The lower bound of the shard range.
-        :param upper_bound: The upper bound of the shard range.
-        """
-        #If the underlying sharding specification is a RANGE, and the
-        #split value is not given, then calculate it as the mid value
-        #between the current lower_bound and its next lower_bound.
-        lower_bound = int(lower_bound)
-        upper_bound = int(upper_bound)
-        split_value = lower_bound + (upper_bound - lower_bound) / 2
-        return split_value
-
-    @staticmethod
-    def is_valid_split_value(split_value, lower_bound, upper_bound):
-        """Verify if the given split value for a RANGE sharding definition is
-        valid.
-
-        :param split_value: The split value.
-        :param lower_bound: The lower bound of the sharding definition.
-        :param upper_bound: The upper bound of the sharding definition.
-
-        :return: True - If the split value is valid
-                False - If the split value is not valid
-        """
-        #A split value needs to be a valid lower_bound value and should
-        #lie between the lower bound and the upper bound values
-        if not RangeShardingIntegerHandler.is_valid_lower_bound(split_value):
-            return False
-        if (int(split_value) > int(lower_bound)) is not True:
-            return False
-        if upper_bound is not None:
-            if (int(split_value) < int(upper_bound)) is not True:
-                return False
-        return True
-
-class HashShardingIntegerHandler(object):
-    """Contains the members that are required to handle a hash based
-    sharding definition.
-    """
-    #Fetch the shard ID corresponding to the input key.
-    LOOKUP_KEY = (
-                "("
-                "SELECT "
-                "sr.shard_mapping_id, "
-                "HEX(sr.lower_bound) AS lower_bound, "
-                "s.shard_id "
-                "FROM shard_ranges AS sr, shards AS s "
-                "WHERE MD5(%s) >= HEX(sr.lower_bound) "
-                "AND sr.shard_mapping_id = %s "
-                "AND s.shard_id = sr.shard_id "
-                "ORDER BY HEX(sr.lower_bound) DESC "
-                "LIMIT 1"
-                ") "
-                "UNION ALL "
-                "("
-                "SELECT "
-                "sr.shard_mapping_id, "
-                "HEX(sr.lower_bound) AS lower_bound, "
-                "sr.shard_id "
-                "FROM shard_ranges AS sr, shards AS s "
-                "WHERE sr.shard_mapping_id = %s "
-                "AND s.shard_id = sr.shard_id "
-                "ORDER BY HEX(sr.lower_bound) DESC "
-                "LIMIT 1"
-                ") "
-                "ORDER BY HEX(lower_bound) ASC "
-                "LIMIT 1"
-                )
-
-    #Select the UPPER BOUND for a given LOWER BOUND value.
-    SELECT_UPPER_BOUND = (
-        "SELECT HEX(lower_bound) FROM "
-        "shard_ranges "
-        "WHERE "
-        "HEX(lower_bound) > %s "
-        "AND "
-        "shard_mapping_id = %s "
-        "ORDER BY HEX(lower_bound) ASC LIMIT 1"
-    )
-
-    @staticmethod
-    def is_valid_lower_bound(lower_bound):
-        """Lower bounds in hash based sharding are autogenerated.
-        This method is thus a no-op for hash based sharding.
-        """
-        return False
-
-    @staticmethod
-    def split_value(lower_bound, upper_bound):
-        """Calculate the split value for a hash based shard as the mid-point
-        of the lower and upper bounds of the current shard.
-
-        :param lower_bound: The lower bound of the hash based shard.
-        :param upper_bound: The upper bound of the hash based shard.
-
-        :return: Hexadecimal representatin of the mid values.
-        """
-        #Retrieve an integer representation of the hexadecimal
-        #lower_bound. The value is actually a long. Python automatically
-        #returns a long value.
-        lower_bound = int(lower_bound,  16)
-        upper_bound = int(upper_bound, 16)
-        #split value after the below computation is actually a long.
-        split_value = lower_bound + (upper_bound - lower_bound) / 2
-        #split_value after the hex computation gets stored with a prefix
-        #0x indicating a hexadecimal value and a suffix of L indicating a
-        #Long. Extract the hexadecimal string from this value.
-        split_value = "%x" % (split_value)
-        return split_value
-
-    @staticmethod
-    def is_valid_split_value(split_value, lower_bound, upper_bound):
-        return True
 
 class RangeShardingSpecification(_persistence.Persistable):
     """Represents a RANGE sharding specification. The class helps encapsulate
@@ -1017,7 +865,9 @@ class RangeShardingSpecification(_persistence.Persistable):
                                 "lower_bound VARBINARY(16) NOT NULL, "
                                 "INDEX(lower_bound), "
                                 "UNIQUE(shard_mapping_id, lower_bound), "
-                                "shard_id INT NOT NULL)")
+                                "shard_id INT NOT NULL) "
+                                "DEFAULT CHARSET=utf8"
+    )
 
     #Create the referential integrity constraint with the shard_mapping_defn
     #table
@@ -1072,18 +922,6 @@ class RangeShardingSpecification(_persistence.Persistable):
         " WHERE shard_id = %s"
     )
 
-    PRUNE_LIMIT = 10000
-
-    PRUNE_WITH_UPPER_BOUND = (
-        "DELETE FROM %s WHERE %s < %s OR %s >= %s LIMIT {prune_limit}".\
-            format(prune_limit=PRUNE_LIMIT)
-    )
-
-    PRUNE_WITHOUT_UPPER_BOUND = (
-        "DELETE FROM %s WHERE %s < %s LIMIT {prune_limit}".\
-            format(prune_limit=PRUNE_LIMIT)
-    )
-
     def __init__(self, shard_mapping_id, lower_bound, shard_id):
         """Initialize a given RANGE sharding mapping specification.
 
@@ -1180,8 +1018,7 @@ class RangeShardingSpecification(_persistence.Persistable):
 
         cur = persister.exec_stmt(
                     RangeShardingSpecification.LIST_RANGE_SPECIFICATION,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (shard_mapping_id,)})
         rows = cur.fetchall()
         return [ RangeShardingSpecification(*row[0:5]) for row in rows ]
@@ -1194,11 +1031,13 @@ class RangeShardingSpecification(_persistence.Persistable):
                           state store.
         """
         persister.exec_stmt(
-            RangeShardingSpecification.ADD_FOREIGN_KEY_CONSTRAINT_SHARD_MAPPING_ID
-            )
+            RangeShardingSpecification.
+            ADD_FOREIGN_KEY_CONSTRAINT_SHARD_MAPPING_ID
+        )
         persister.exec_stmt(
-            RangeShardingSpecification.ADD_FOREIGN_KEY_CONSTRAINT_SHARD_ID
-            )
+            RangeShardingSpecification.
+            ADD_FOREIGN_KEY_CONSTRAINT_SHARD_ID
+        )
 
     @staticmethod
     def fetch(shard_id, persister=None):
@@ -1214,8 +1053,7 @@ class RangeShardingSpecification(_persistence.Persistable):
         """
         cur = persister.exec_stmt(
                     RangeShardingSpecification.SELECT_RANGE_SPECIFICATION,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (shard_id,)})
         row = cur.fetchone()
         if row is None:
@@ -1249,8 +1087,7 @@ class RangeShardingSpecification(_persistence.Persistable):
                 which the key belongs.
         """
         cur = persister.exec_stmt(SHARDING_DATATYPE_HANDLER[type].LOOKUP_KEY,
-                    {"raw" : False,
-                    "fetch" : False,
+                    {"fetch" : False,
                     "params" : (key, shard_mapping_id)})
 
         row = cur.fetchone()
@@ -1275,8 +1112,7 @@ class RangeShardingSpecification(_persistence.Persistable):
         """
         cur = persister.exec_stmt(
                         SHARDING_DATATYPE_HANDLER[type].SELECT_UPPER_BOUND,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (lower_bound, shard_mapping_id)})
 
         row = cur.fetchone()
@@ -1287,7 +1123,7 @@ class RangeShardingSpecification(_persistence.Persistable):
         return row[0]
 
     @staticmethod
-    def delete_from_shard_db(table_name, type_name):
+    def delete_from_shard_db(table_name, type_name, prune_limit):
         """Delete the data from the copied data directories based on the
         sharding configuration uploaded in the sharding tables of the state
         store. The basic logic consists of
@@ -1301,6 +1137,9 @@ class RangeShardingSpecification(_persistence.Persistable):
           server.
 
         :param table_name: The table being sharded.
+        :param type_name: The type of the sharding definition.
+        :param prune_limit: The number of DELETEs that should be
+                            done in one batch.
         """
 
         shard_mapping = ShardMapping.fetch(table_name)
@@ -1315,15 +1154,19 @@ class RangeShardingSpecification(_persistence.Persistable):
                                                          " shard mapping ID.")
 
         for shard in shards:
-            RangeShardingSpecification.prune_shard_id(shard.shard_id, type_name)
+            RangeShardingSpecification.prune_shard_id(shard.shard_id,
+                                                      type_name, prune_limit)
 
     @staticmethod
-    def prune_shard_id(shard_id, type_name):
+    def prune_shard_id(shard_id, type_name, prune_limit):
         """Remove the rows in the shard that do not match the metadata
         in the shard_range tables. When the rows are being removed
         foreign key checks will be disabled.
 
         :param shard_id: The ID of the shard that needs to be pruned.
+        :param type_name: The type of the sharding definition.
+        :param prune_limit: The number of DELETEs that should be
+                            done in one batch.
         """
 
         range_sharding_spec = RangeShardingSpecification.fetch(shard_id)
@@ -1350,20 +1193,24 @@ class RangeShardingSpecification(_persistence.Persistable):
             table_name = shard_mapping.table_name
 
             if upper_bound is not None:
-                delete_query = \
-                RangeShardingSpecification.PRUNE_WITH_UPPER_BOUND % (
+                delete_query = (
+                    SHARDING_DATATYPE_HANDLER[type_name].\
+                        PRUNE_SHARD_WITH_UPPER_BOUND) % (
                     table_name,
                     shard_mapping.column_name,
                     range_sharding_spec.lower_bound,
                     shard_mapping.column_name,
-                    upper_bound
+                    upper_bound,
+                    prune_limit
                 )
             else:
-                delete_query = \
-                RangeShardingSpecification.PRUNE_WITHOUT_UPPER_BOUND % (
+                delete_query = (
+                    SHARDING_DATATYPE_HANDLER[type_name].\
+                        PRUNE_SHARD_WITHOUT_UPPER_BOUND) % (
                     table_name,
                     shard_mapping.column_name,
-                    range_sharding_spec.lower_bound
+                    range_sharding_spec.lower_bound,
+                    prune_limit
                 )
 
             shard = Shards.fetch(range_sharding_spec.shard_id)
@@ -1393,16 +1240,14 @@ class RangeShardingSpecification(_persistence.Persistable):
             #failing. Hence we need to disable foreign key checks during the
             #pruning process.
             master.set_foreign_key_checks(False)
-            deleted = RangeShardingSpecification.PRUNE_LIMIT
-            while deleted == RangeShardingSpecification.PRUNE_LIMIT:
+            prune_limit = int(prune_limit)
+            deleted = prune_limit
+            while deleted == prune_limit:
                 delete_cursor = master.exec_stmt(
-                     delete_query,
-                     {
-                         "raw":False,
-                         "fetch":False
-                     }
+                    delete_query, {"fetch":False}
                 )
                 deleted = delete_cursor.rowcount
+            #Enable Foreign Key Checking
             master.set_foreign_key_checks(True)
 
 class HashShardingSpecification(RangeShardingSpecification):
@@ -1460,18 +1305,6 @@ class HashShardingSpecification(RangeShardingSpecification):
         "WHERE shard_mapping_id = %s"
         )
 
-    PRUNE_WITH_UPPER_BOUND = (
-        "DELETE FROM %s WHERE MD5(%s) < '%s' OR MD5(%s) >= '%s' "
-        "LIMIT {prune_limit}".
-        format(prune_limit=RangeShardingSpecification.PRUNE_LIMIT)
-    )
-
-    PRUNE_WITHOUT_UPPER_BOUND = (
-        "DELETE FROM %s WHERE MD5(%s) >= '%s' AND MD5(%s) < '%s' "
-        "LIMIT {prune_limit}".
-        format(prune_limit=RangeShardingSpecification.PRUNE_LIMIT)
-    )
-
     def __init__(self, shard_mapping_id, lower_bound, shard_id):
         """Initialize a given HASH sharding mapping specification.
 
@@ -1516,7 +1349,7 @@ class HashShardingSpecification(RangeShardingSpecification):
             raise _errors.ShardingError("Shard Mapping not found.")
 
         for shard_mapping in shard_mappings:
-            max_query = "SELECT HEX(MD5(MAX(%s))) FROM %s" % \
+            max_query = "SELECT MD5(MAX(%s)) FROM %s" % \
                         (
                         shard_mapping.column_name,
                         shard_mapping.table_name
@@ -1545,17 +1378,12 @@ class HashShardingSpecification(RangeShardingSpecification):
 
             master.connect()
 
-            cur = master.exec_stmt(
-                            max_query,
-                            {"raw" : False,
-                            "fetch" : False,
-                            }
-                        )
+            cur = master.exec_stmt(max_query, {"fetch" : False})
 
             row = cur.fetchone()
 
             if row is not None:
-                max_keys.append(row[0])
+                max_keys.append(str(row[0]))
 
         #max_keys stores  all the maximum values in all the tables. We will
         #fetch the maximum values among all these values and use it as the
@@ -1627,7 +1455,6 @@ class HashShardingSpecification(RangeShardingSpecification):
                 which the key belongs.
         """
         cur = persister.exec_stmt(SHARDING_DATATYPE_HANDLER[type].LOOKUP_KEY, {
-                        "raw" : False,
                         "fetch" : False,
                         "params" : (
                             key,
@@ -1655,8 +1482,7 @@ class HashShardingSpecification(RangeShardingSpecification):
         """
         cur = persister.exec_stmt(
                     HashShardingSpecification.SELECT_RANGE_SPECIFICATION,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (shard_id,)})
         row = cur.fetchone()
         if row is None:
@@ -1674,8 +1500,7 @@ class HashShardingSpecification(RangeShardingSpecification):
         """
         cur = persister.exec_stmt(
                     HashShardingSpecification.SELECT_LEAST_LOWER_BOUND,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (shard_mapping_id,)})
         row = cur.fetchone()
         if row is None:
@@ -1696,8 +1521,7 @@ class HashShardingSpecification(RangeShardingSpecification):
         """
         cur = persister.exec_stmt(
                     HashShardingSpecification.LIST_RANGE_SPECIFICATION,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (shard_mapping_id,)})
         rows = cur.fetchall()
         return [ HashShardingSpecification(*row[0:5]) for row in rows ]
@@ -1724,7 +1548,7 @@ class HashShardingSpecification(RangeShardingSpecification):
         pass
 
     @staticmethod
-    def delete_from_shard_db(table_name, type_name):
+    def delete_from_shard_db(table_name, type_name, prune_limit):
         """Delete the data from the copied data directories based on the
         sharding configuration uploaded in the sharding tables of the state
         store. The basic logic consists of
@@ -1738,6 +1562,9 @@ class HashShardingSpecification(RangeShardingSpecification):
           server.
 
         :param table_name: The table being sharded.
+        :param type_name: The type of the sharding definition.
+        :param prune_limit: The number of DELETEs that should be
+                            done in one batch.
         """
 
         shard_mapping = ShardMapping.fetch(table_name)
@@ -1750,9 +1577,9 @@ class HashShardingSpecification(RangeShardingSpecification):
         if not shards:
             raise _errors.ShardingError("No shards associated with this"
                                         " shard mapping ID.")
-
         for shard in shards:
-            HashShardingSpecification.prune_shard_id(shard.shard_id, type_name)
+            HashShardingSpecification.prune_shard_id(shard.shard_id, type_name,
+                                                     prune_limit)
 
     @staticmethod
     def get_upper_bound(lower_bound, shard_mapping_id, type, persister=None):
@@ -1770,8 +1597,7 @@ class HashShardingSpecification(RangeShardingSpecification):
         """
         cur = persister.exec_stmt(
                         SHARDING_DATATYPE_HANDLER[type].SELECT_UPPER_BOUND,
-                        {"raw" : False,
-                        "fetch" : False,
+                        {"fetch" : False,
                         "params" : (lower_bound, shard_mapping_id)})
 
         row = cur.fetchone()
@@ -1782,11 +1608,14 @@ class HashShardingSpecification(RangeShardingSpecification):
         return row[0]
 
     @staticmethod
-    def prune_shard_id(shard_id, type_name):
+    def prune_shard_id(shard_id, type_name, prune_limit):
         """Remove the rows in the shard that do not match the metadata
         in the shard_range tables.
 
         :param shard_id: The ID of the shard that needs to be pruned.
+        :param type_name: The type of the sharding definition.
+        :param prune_limit: The number of DELETEs that should be
+                            done in one batch.
         """
 
         hash_sharding_spec = HashShardingSpecification.fetch(shard_id)
@@ -1811,29 +1640,35 @@ class HashShardingSpecification(RangeShardingSpecification):
             table_name = shard_mapping.table_name
 
             if upper_bound is not None:
-                delete_query = \
-                    HashShardingSpecification.PRUNE_WITH_UPPER_BOUND  % (
+                delete_query = (
+                    SHARDING_DATATYPE_HANDLER[type_name].\
+                        PRUNE_SHARD_WITH_UPPER_BOUND
+                ) % (
                     table_name,
                     shard_mapping.column_name,
                     hash_sharding_spec.lower_bound,
                     shard_mapping.column_name,
-                    upper_bound
-                    )
+                    upper_bound,
+                    prune_limit
+                )
             else:
                 #HASH based sharding forms a circular ring. Hence
                 #when there is no upper_bound, all the values, that
                 #circle around from the largest lower_bound to the
                 #least upper_bound need to be present in this shard.
-                delete_query = \
-                    HashShardingSpecification.PRUNE_WITHOUT_UPPER_BOUND % (
+                delete_query = (
+                    SHARDING_DATATYPE_HANDLER[type_name].\
+                        PRUNE_SHARD_WITHOUT_UPPER_BOUND
+                ) % (
                     table_name,
                     shard_mapping.column_name,
                     HashShardingSpecification.fetch_least_lower_bound(
                         shard_mapping.shard_mapping_id
                     ),
                     shard_mapping.column_name,
-                    hash_sharding_spec.lower_bound
-                    )
+                    hash_sharding_spec.lower_bound,
+                    prune_limit
+                )
 
             shard = Shards.fetch(hash_sharding_spec.shard_id)
             if shard is None:
@@ -1858,17 +1693,18 @@ class HashShardingSpecification(RangeShardingSpecification):
 
             master.connect()
 
+            #Dependencies between tables being pruned may lead to the prune
+            #failing. Hence we need to disable foreign key checks during the
+            #pruning process.
             master.set_foreign_key_checks(False)
-            deleted = RangeShardingSpecification.PRUNE_LIMIT
-            while deleted == RangeShardingSpecification.PRUNE_LIMIT:
+            prune_limit = int(prune_limit)
+            deleted = prune_limit
+            while deleted == prune_limit:
                 delete_cursor = master.exec_stmt(
-                     delete_query,
-                     {
-                         "raw":False,
-                         "fetch":False
-                     }
+                    delete_query, {"fetch":False}
                 )
                 deleted = delete_cursor.rowcount
+            #Enable the Foreign Key checks after the prune.
             master.set_foreign_key_checks(True)
 
 class MappingShardsGroups(_persistence.Persistable):
@@ -1951,16 +1787,22 @@ class MappingShardsGroups(_persistence.Persistable):
         assert(criterion in ("shard_id", "shard_mapping_id", "table_name"))
         rows = persister.exec_stmt(
             MappingShardsGroups.PARAM_QUERIES[locality][criterion],
-            {"raw" : False, "fetch" : True, "params" : (value, )}
+            {"params" : (value, )}
         )
         return rows
 
 SHARDING_DATATYPE_HANDLER = {
-    "RANGE":RangeShardingIntegerHandler,
-    "HASH":HashShardingIntegerHandler
+    "RANGE": RangeShardingIntegerHandler,
+    "RANGE_INTEGER": RangeShardingIntegerHandler,
+    "RANGE_STRING": RangeShardingStringHandler,
+    "RANGE_DATETIME": RangeShardingDateTimeHandler,
+    "HASH": HashShardingHandler
 }
 
 SHARDING_SPECIFICATION_HANDLER = {
-    "RANGE":RangeShardingSpecification,
-    "HASH":HashShardingSpecification
+    "RANGE": RangeShardingSpecification,
+    "RANGE_INTEGER": RangeShardingSpecification,
+    "RANGE_STRING": RangeShardingSpecification,
+    "RANGE_DATETIME": RangeShardingSpecification,
+    "HASH": HashShardingSpecification
 }
